import lodash from 'lodash'
import { interact, replaceApiParams } from '../../utils'
import apis from '../../config/api'

export default {
  namespaced: true,
  state: {
    persons: [],
    detail: {},
    visitingLog: [],
    guards: [],
  },
  mutations: {
    updateList (state, payload) {
      const { records } = payload
      state.persons = records
    },
    mergeList (state, payload) {
      state.persons = state.persons.map((person) => {
        return person.personId === payload.personId ? { ...payload } : person
      })
    },
    updateGuards (state, payload) {
      const { records } = payload
      state.guards = records
    },
    updateDetail (state, payload) {
      state.detail = payload.detail || {}
      // state.visitingLog = payload.visitingLog || []
    },
    updateLog (state, payload) {
      state.visitingLog = (state.visitingLog || []).map((log) => {
        return log.logId === payload.logId ? { ...payload } : log
      })
    },
    resetVisitingLog (state, payload) {
      state.visitingLog = payload || []
    },
  },
  actions: {
    setDetailNull ({ commit }) {
      commit('updateDetail', {})
    },
    getGuards ({ commit }) {
      return new Promise((resolve, reject) => {
        const api = lodash.get(apis, 'guard.device')
        interact(api).then((data) => {
          commit('updateGuards', { records: data })
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    getPersonList ({ commit }, payload) {
      const { typeId } = payload
      return new Promise((resolve, reject) => {
        const api = lodash.get(apis, 'person.list')
        interact(api, payload).then((data) => {
          const { records, ...pageNavi } = data;

          commit('updateList', { typeId, pageNavi, records })

          resolve({ records, pageNavi })
        }).catch(() => {
          reject()
        })
      })
    },
    getVisitingLog ({ commit }, payload) {
      return new Promise((resolve, reject) => {
        const api = lodash.get(apis, 'visitinglog.list')
        interact(api, payload).then((data = {}) => {
          commit('resetVisitingLog', data.records || [])
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    getDetail ({ commit }, payload) {
      const api = replaceApiParams(lodash.get(apis, 'person.detail'), payload)
      interact(api).then((data) => {
        commit('updateDetail', data)
      })
    },
    getPurPerson (ctx, payload) {
      const api = replaceApiParams(lodash.get(apis, 'person.detail'), payload)
      return interact(api);
    },
    mergePerson ({ commit }, payload) {
      const { from, to } = payload
      const api = replaceApiParams(lodash.get(apis, 'person.merge'), { id: from })
      return interact(api, JSON.stringify(to)).then(() => {
        const person = { ...to, personId: from }
        commit('mergeList', person)
      })
    },
    addPerson ( _, payload) {
      const { onSuccess } = payload
      const api = lodash.get(apis, 'person.edi.vue')
      interact(api, payload.detail).then((data) => {
        onSuccess(data)
      })
    },
    editPerson ( _, payload) {
      const { onSuccess } = payload
      const api = lodash.get(apis, 'person.edit')
      interact(api, payload.detail).then((data) => {
        onSuccess(data)
      })
    },
    editVisibleLog (_, payload) {
      return new Promise((resolve, reject) => {
        const api = replaceApiParams(lodash.get(apis, 'log.edit'), payload)
        interact(api, payload.detail).then((data) => {
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    dispatchToHik (_, payload) {
      return new Promise((resolve, reject) => {
        const api = replaceApiParams(lodash.get(apis, 'log.dispatch'), payload)
        interact(api).then((data) => {
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    dispatchFace (_, payload) {
      return new Promise((resolve, reject) => {
        const api = replaceApiParams(lodash.get(apis, 'person.dispatch'), payload)
        interact(api).then((data) => {
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    deletePerson ( _, payload) {
      return new Promise((resolve, reject) => {
        const api = replaceApiParams(lodash.get(apis, 'person.delete'), payload)
        interact(api).then((data) => {
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    dispatchGuard (_, payload) {
      const { id, faceId } = payload || {}

      return new Promise((resolve, reject) => {
        const api = replaceApiParams(lodash.get(apis, 'guard.dispatch'), { id })
        interact({ ...api, params: { faceId }  }).then((data) => {
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    },
    forbidGuard (_, payload) {
      return new Promise((resolve, reject) => {
        const api = replaceApiParams(lodash.get(apis, 'guard.forbid'), payload)
        interact(api).then((data) => {
          resolve(data)
        }).catch(() => {
          reject()
        })
      })
    }
  }
}
